@@ -92,6 +92,8 @@ from debputy.util import (
     escape_shell,
     program_name,
     integrated_with_debhelper,
+    change_log_level,
+    PRINT_COMMAND,
 )
 
 
@@ -652,6 +654,16 @@ def _run_tests_for_plugin(context: CommandContext) -> None:
 def _dh_integration_generate_debs(context: CommandContext) -> None:
     integrated_with_debhelper()
     parsed_args = context.parsed_args
+    log_level: Optional[int] = None
+    if os.environ.get("DH_VERBOSE", "") != "":
+        log_level = PRINT_COMMAND
+    if parsed_args.debug_mode:
+        log_level = logging.INFO
+    if log_level is not None:
+        _warn(
+            f"LOG LEVEL: {log_level} -- {logging.WARNING} -- {PRINT_COMMAND} -- {logging.INFO}"
+        )
+        change_log_level(log_level)
     integration_mode = context.resolve_integration_mode()
     is_dh_rrr_only_mode = integration_mode == INTEGRATION_MODE_DH_DEBPUTY_RRR
     if is_dh_rrr_only_mode:
@@ -662,9 +674,6 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
             _error(
                 f"Plugins are not supported in the zz-debputy-rrr sequence. Detected plugins: {plugin_names}"
             )
-    debug_materialization = (
-        os.environ.get("DH_VERBOSE", "") != "" or parsed_args.debug_mode
-    )
 
     plugins = context.load_plugins().plugin_data
     for plugin in plugins.values():
@@ -733,7 +742,7 @@ def _dh_integration_generate_debs(context: CommandContext) -> None:
         manifest,
         package_data_table,
         is_dh_rrr_only_mode,
-        debug_materialization=debug_materialization,
+        debug_materialization=log_level is not None,
     )
 
 
