@@ -56,6 +56,7 @@ from .plugin.api.impl_types import (
     PackageContextData,
 )
 from .plugin.api.feature_set import PluginProvidedFeatureSet
+from .plugin.api.spec import DebputyIntegrationMode
 from .yaml import YAMLError, MANIFEST_YAML
 
 try:
@@ -116,6 +117,7 @@ class HighLevelManifestParser(ParserContextData):
         dpkg_arch_query_table: DpkgArchTable,
         build_env: DebBuildOptionsAndProfiles,
         plugin_provided_feature_set: PluginProvidedFeatureSet,
+        debputy_integration_mode: DebputyIntegrationMode,
         *,
         # Available for testing purposes only
         debian_dir: Union[str, VirtualPath] = "./debian",
@@ -132,6 +134,7 @@ class HighLevelManifestParser(ParserContextData):
         self._build_env = build_env
         self._package_state_stack: List[PackageTransformationDefinition] = []
         self._plugin_provided_feature_set = plugin_provided_feature_set
+        self._debputy_integration_mode = debputy_integration_mode
         self._declared_variables = {}
 
         if isinstance(debian_dir, str):
@@ -314,6 +317,14 @@ class HighLevelManifestParser(ParserContextData):
     def is_in_binary_package_state(self) -> bool:
         return bool(self._package_state_stack)
 
+    @property
+    def debputy_integration_mode(self) -> DebputyIntegrationMode:
+        return self._debputy_integration_mode
+
+    @debputy_integration_mode.setter
+    def debputy_integration_mode(self, new_value: DebputyIntegrationMode) -> None:
+        self._debputy_integration_mode = new_value
+
     def _transform_dpkg_maintscript_helpers_to_snippets(self) -> None:
         package_state = self.current_binary_package_state
         for dmh in package_state.dpkg_maintscript_helper_snippets:
