@@ -23,6 +23,7 @@ from typing import (
     Literal,
     Set,
     Iterator,
+    Container,
 )
 from weakref import ref
 
@@ -39,7 +40,7 @@ from debputy.maintscript_snippet import DpkgMaintscriptHelperCommand
 from debputy.manifest_conditions import ManifestCondition
 from debputy.manifest_parser.base_types import DebputyParsedContent, TypeMapping
 from debputy.manifest_parser.exceptions import ManifestParseException
-from debputy.manifest_parser.util import AttributePath
+from debputy.manifest_parser.util import AttributePath, check_integration_mode
 from debputy.packages import BinaryPackage
 from debputy.plugin.api import (
     VirtualPath,
@@ -59,6 +60,7 @@ from debputy.plugin.api.spec import (
     reference_documentation,
     PackagerProvidedFileReferenceDocumentation,
     TypeMappingDocumentation,
+    DebputyIntegrationMode,
 )
 from debputy.substitution import VariableContext
 from debputy.transformation_rules import TransformationRule
@@ -281,6 +283,12 @@ class DeclarativeInputParser(Generic[TD]):
     def inline_reference_documentation(self) -> Optional[ParserDocumentation]:
         return None
 
+    @property
+    def expected_debputy_integration_mode(
+        self,
+    ) -> Optional[Container[DebputyIntegrationMode]]:
+        return None
+
     @property
     def reference_documentation_url(self) -> Optional[str]:
         doc = self.inline_reference_documentation
@@ -297,16 +305,30 @@ class DeclarativeInputParser(Generic[TD]):
 
 
 class DelegatingDeclarativeInputParser(DeclarativeInputParser[TD]):
-    __slots__ = ("delegate", "_reference_documentation")
+    __slots__ = (
+        "delegate",
+        "_reference_documentation",
+        "_expected_debputy_integration_mode",
+    )
 
     def __init__(
         self,
         delegate: DeclarativeInputParser[TD],
         *,
         inline_reference_documentation: Optional[ParserDocumentation] = None,
+        expected_debputy_integration_mode: Optional[
+            Container[DebputyIntegrationMode]
+        ] = None,
     ) -> None:
         self.delegate = delegate
         self._reference_documentation = inline_reference_documentation
+        self._expected_debputy_integration_mode = expected_debputy_integration_mode
+
+    @property
+    def expected_debputy_integration_mode(
+        self,
+    ) -> Optional[Container[DebputyIntegrationMode]]:
+        return self._expected_debputy_integration_mode
 
     @property
     def inline_reference_documentation(self) -> Optional[ParserDocumentation]:
@@ -334,6 +356,9 @@ class ListWrappedDeclarativeInputParser(DelegatingDeclarativeInputParser[TD]):
         *,
         parser_context: Optional["ParserContextData"] = None,
     ) -> TD:
+        check_integration_mode(
+            path, parser_context, self._expected_debputy_integration_mode
+        )
         if not isinstance(value, list):
             doc_ref = self._doc_url_error_suffix(see_url_version=True)
             raise ManifestParseException(
@@ -466,12 +491,22 @@ class DispatchingObjectParser(
         manifest_attribute_path_template: str,
         *,
         parser_documentation: Optional[ParserDocumentation] = None,
+        expected_debputy_integration_mode: Optional[
+            Container[DebputyIntegrationMode]
+        ] = None,
     ) -> None:
         super().__init__(manifest_attribute_path_template)
         self._attribute_documentation: List[ParserAttributeDocumentation] = []
         if parser_documentation is None:
             parser_documentation = reference_documentation()
         self._parser_documentation = parser_documentation
+        self._expected_debputy_integration_mode = expected_debputy_integration_mode
+
+    @property
+    def expected_debputy_integration_mode(
+        self,
+    ) -> Optional[Container[DebputyIntegrationMode]]:
+        return self._expected_debputy_integration_mode
 
     @property
     def reference_documentation_url(self) -> Optional[str]:
@@ -540,6 +575,11 @@ class DispatchingObjectParser(
         *,
         parser_context: "ParserContextData",
     ) -> TP:
+        check_integration_mode(
+            attribute_path,
+            parser_context,
+            self._expected_debputy_integration_mode,
+        )
         doc_ref = ""
         if self.reference_documentation_url is not None:
             doc_ref = (
@@ -602,6 +642,8 @@ class PackageContextData(Generic[TP]):
 class InPackageContextParser(
     DelegatingDeclarativeInputParser[Mapping[str, PackageContextData[TP]]]
 ):
+    __slots__ = ()
+
     def __init__(
         self,
         manifest_attribute_path_template: str,
@@ -621,6 +663,11 @@ class InPackageContextParser(
         parser_context: Optional["ParserContextData"] = None,
     ) -> TP:
         assert parser_context is not None
+        check_integration_mode(
+            attribute_path,
+            parser_context,
+            self._expected_debputy_integration_mode,
+        )
         doc_ref = ""
         if self.reference_documentation_url is not None:
             doc_ref = (
