@@ -105,6 +105,7 @@ _PROFILE_GROUP_SPLIT = re.compile(r">\s+<")
 _DEFAULT_LOGGER: Optional[logging.Logger] = None
 _STDOUT_HANDLER: Optional[logging.StreamHandler[Any]] = None
 _STDERR_HANDLER: Optional[logging.StreamHandler[Any]] = None
+PRINT_COMMAND = logging.INFO + 5
 
 
 def assume_not_none(x: Optional[T]) -> T:
@@ -225,8 +226,8 @@ def escape_shell(*args: str) -> str:
     return " ".join(_escape_shell_word(w) for w in args)
 
 
-def print_command(*args: str, print_at_log_level: int = logging.INFO) -> None:
-    if logging.getLogger().isEnabledFor(print_at_log_level):
+def print_command(*args: str, print_at_log_level: int = PRINT_COMMAND) -> None:
+    if _DEFAULT_LOGGER and _DEFAULT_LOGGER.isEnabledFor(print_at_log_level):
         print(f"   {escape_shell(*args)}")
 
 
@@ -728,6 +729,13 @@ def package_cross_check_precheck(
     return a_may_see_b, b_may_see_a
 
 
+def change_log_level(
+    log_level: int,
+) -> None:
+    if _DEFAULT_LOGGER is not None:
+        _DEFAULT_LOGGER.setLevel(log_level)
+
+
 def setup_logging(
     *,
     log_only_to_stderr: bool = False,
